@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005, 2006 Christian Schoenebeck                        *
+ *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,8 +23,13 @@
 
 #include "MidiInputPort.h"
 
+#include "../../common/global_private.h"
+#include "MidiInstrumentMapper.h"
 #include "../../Sampler.h"
 #include "../../engines/EngineFactory.h"
+#include "VirtualMidiDevice.h"
+
+#include <algorithm>
 
 namespace LinuxSampler {
 
@@ -71,11 +76,11 @@ namespace LinuxSampler {
 
     MidiInputPort::MidiInputPort(MidiInputDevice* pDevice, int portNumber)
         : MidiChannelMapReader(MidiChannelMap),
-          SysexListenersReader(SysexListeners) {
+          SysexListenersReader(SysexListeners),
+          virtualMidiDevicesReader(virtualMidiDevices) {
         this->pDevice = pDevice;
         this->portNumber = portNumber;
         Parameters["NAME"] = new ParameterName(this);
-        pPreviousProgramChangeEngineChannel = NULL;
     }
 
     MidiInputDevice* MidiInputPort::GetDevice() {
@@ -106,6 +111,13 @@ namespace LinuxSampler {
             for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity);
         }
         MidiChannelMapReader.Unlock();
+
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendNoteOnToDevice(Key, Velocity);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchNoteOn(uint8_t Key, uint8_t Velocity, uint MidiChannel, int32_t FragmentPos) {
@@ -124,6 +136,13 @@ namespace LinuxSampler {
             for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
+
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendNoteOnToDevice(Key, Velocity);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchNoteOff(uint8_t Key, uint8_t Velocity, uint MidiChannel) {
@@ -142,6 +161,13 @@ namespace LinuxSampler {
             for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity);
         }
         MidiChannelMapReader.Unlock();
+
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendNoteOffToDevice(Key, Velocity);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchNoteOff(uint8_t Key, uint8_t Velocity, uint MidiChannel, int32_t FragmentPos) {
@@ -160,6 +186,13 @@ namespace LinuxSampler {
             for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
+
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendNoteOffToDevice(Key, Velocity);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchPitchbend(int Pitch, uint MidiChannel) {
@@ -239,7 +272,7 @@ namespace LinuxSampler {
         // dispatch event to all engine instances
         std::set<Engine*>::iterator engineiter = allEngines.begin();
         std::set<Engine*>::iterator end        = allEngines.end();
-        for (; engineiter != end; engineiter++) (*engineiter)->SendSysex(pData, Size);
+        for (; engineiter != end; engineiter++) (*engineiter)->SendSysex(pData, Size, this);
         SysexListenersReader.Unlock();
     }
 
@@ -251,23 +284,136 @@ namespace LinuxSampler {
             return;
         }
 
-        Sampler*        pSampler        = (Sampler*) pDevice->pSampler;
-        SamplerChannel* pSamplerChannel = pSampler->GetSamplerChannel(Program);
-        if (!pSamplerChannel) return;
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendProgramChange(Program);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendProgramChange(Program);
+        }
+        MidiChannelMapReader.Unlock();
+    }
 
-        EngineChannel* pEngineChannel = pSamplerChannel->GetEngineChannel();
-        if (!pEngineChannel) return;
+    void MidiInputPort::DispatchBankSelectMsb(uint8_t BankMSB, uint MidiChannel) {
+        if (BankMSB > 127 || MidiChannel > 16) return;
+        if (!pDevice || !pDevice->pSampler) {
+            std::cerr << "MidiInputPort: ERROR, no sampler instance to handle bank select MSB."
+                      << "This is a bug, please report it!\n" << std::flush;
+            return;
+        }
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            // according to the MIDI specs, a bank select should not alter the patch
+            for (; engineiter != end; engineiter++) (*engineiter)->SetMidiBankMsb(BankMSB);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            // according to the MIDI specs, a bank select should not alter the patch
+            for (; engineiter != end; engineiter++) (*engineiter)->SetMidiBankMsb(BankMSB);
+        }
+        MidiChannelMapReader.Unlock();
+    }
 
-        // disconnect from the engine channel which was connected by the last PC event
-        if (pPreviousProgramChangeEngineChannel)
-            Disconnect(pPreviousProgramChangeEngineChannel);
+    void MidiInputPort::DispatchBankSelectLsb(uint8_t BankLSB, uint MidiChannel) {
+        if (BankLSB > 127 || MidiChannel > 16) return;
+        if (!pDevice || !pDevice->pSampler) {
+            std::cerr << "MidiInputPort: ERROR, no sampler instance to handle bank select LSB."
+                      << "This is a bug, please report it!\n" << std::flush;
+            return;
+        }
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            // according to the MIDI specs, a bank select should not alter the patch
+            for (; engineiter != end; engineiter++) (*engineiter)->SetMidiBankLsb(BankLSB);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            // according to the MIDI specs, a bank select should not alter the patch
+            for (; engineiter != end; engineiter++) (*engineiter)->SetMidiBankLsb(BankLSB);
+        }
+        MidiChannelMapReader.Unlock();
+    }
 
-        // now connect to the new engine channel and remember it
-        try {
-            Connect(pEngineChannel, (midi_chan_t) MidiChannel);
-            pPreviousProgramChangeEngineChannel = pEngineChannel;
+    void MidiInputPort::DispatchRaw(uint8_t* pData) {
+        uint8_t channel = pData[0] & 0x0f;
+        switch (pData[0] & 0xf0) {
+        case 0x80:
+            DispatchNoteOff(pData[1], pData[2], channel);
+            break;
+        case 0x90:
+            if (pData[2]) {
+                DispatchNoteOn(pData[1], pData[2], channel);
+            } else {
+                DispatchNoteOff(pData[1], pData[2], channel);
+            }
+            break;
+        case 0xb0:
+            if (pData[1] == 0) {
+                DispatchBankSelectMsb(pData[2], channel);
+            } else if (pData[1] == 32) {
+                DispatchBankSelectLsb(pData[2], channel);
+            }
+            DispatchControlChange(pData[1], pData[2], channel);
+            break;
+        case 0xc0:
+            DispatchProgramChange(pData[1], channel);
+            break;
+        case 0xd0:
+            DispatchControlChange(128, pData[1], channel);
+            break;
+        case 0xe0:
+            DispatchPitchbend((pData[1] | pData[2] << 7) - 8192, channel);
+            break;
+        }
+    }
+
+    void MidiInputPort::DispatchRaw(uint8_t* pData, int32_t FragmentPos) {
+        uint8_t channel = pData[0] & 0x0f;
+        switch (pData[0] & 0xf0) {
+        case 0x80:
+            DispatchNoteOff(pData[1], pData[2], channel, FragmentPos);
+            break;
+        case 0x90:
+            if (pData[2]) {
+                DispatchNoteOn(pData[1], pData[2], channel, FragmentPos);
+            } else {
+                DispatchNoteOff(pData[1], pData[2], channel, FragmentPos);
+            }
+            break;
+        case 0xb0:
+            if (pData[1] == 0) {
+                DispatchBankSelectMsb(pData[2], channel);
+            } else if (pData[1] == 32) {
+                DispatchBankSelectLsb(pData[2], channel);
+            }
+            DispatchControlChange(pData[1], pData[2], channel, FragmentPos);
+            break;
+        case 0xc0:
+            DispatchProgramChange(pData[1], channel);
+            break;
+        case 0xd0:
+            DispatchControlChange(128, pData[1], channel, FragmentPos);
+            break;
+        case 0xe0:
+            DispatchPitchbend((pData[1] | pData[2] << 7) - 8192, channel, FragmentPos);
+            break;
         }
-        catch (...) { /* NOOP */ }
     }
 
     void MidiInputPort::Connect(EngineChannel* pEngineChannel, midi_chan_t MidiChannel) {
@@ -344,4 +490,36 @@ namespace LinuxSampler {
         return count;
     }
 
+    void MidiInputPort::Connect(VirtualMidiDevice* pDevice) {
+        virtualMidiDevicesMutex.Lock();
+        // double buffer ... double work ...
+        {
+            std::vector<VirtualMidiDevice*>& devices =
+                virtualMidiDevices.GetConfigForUpdate();
+            devices.push_back(pDevice);
+        }
+        {
+            std::vector<VirtualMidiDevice*>& devices =
+                virtualMidiDevices.SwitchConfig();
+            devices.push_back(pDevice);
+        }
+        virtualMidiDevicesMutex.Unlock();
+    }
+
+    void MidiInputPort::Disconnect(VirtualMidiDevice* pDevice) {
+        virtualMidiDevicesMutex.Lock();
+        // double buffer ... double work ...
+        {
+            std::vector<VirtualMidiDevice*>& devices =
+                virtualMidiDevices.GetConfigForUpdate();
+            devices.erase(std::find(devices.begin(), devices.end(), pDevice));
+        }
+        {
+            std::vector<VirtualMidiDevice*>& devices =
+                virtualMidiDevices.SwitchConfig();
+            devices.erase(std::find(devices.begin(), devices.end(), pDevice));
+        }
+        virtualMidiDevicesMutex.Unlock();
+    }
+
 } // namespace LinuxSampler
