@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 Christian Schoenebeck                              *
+ *   Copyright (C) 2005 - 2007 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,13 +21,15 @@
  *   MA  02111-1307  USA                                                   *
  ***************************************************************************/
 
+// Note: the assembly code is currently disabled, as it doesn't fit into
+// the new synthesis core introduced by LS 0.4.0
+
 #ifndef __LS_RESAMPLER_H__
 #define __LS_RESAMPLER_H__
 
-#include "../../common/global.h"
+#include "../../common/global_private.h"
 
 // TODO: cubic interpolation is not yet supported by the MMX/SSE(1) version though
-// TODO: cubic interpolation is not supported for 24 bit samples
 #ifndef USE_LINEAR_INTERPOLATION
 # define USE_LINEAR_INTERPOLATION   1  ///< set to 0 if you prefer cubic interpolation (slower, better quality)
 #endif
@@ -75,7 +77,7 @@ namespace LinuxSampler {
                 }
             }
 
-#if CONFIG_ASM && ARCH_X86
+#if 0 // CONFIG_ASM && ARCH_X86
             inline static void GetNext4SamplesMonoMMXSSE(sample_t* pSrc, void* Pos, float& Pitch) {
                 if (INTERPOLATE) Interpolate4StepsMonoMMXSSE(pSrc, Pos, Pitch);
                 else { // no pitch, so no interpolation necessary
@@ -147,11 +149,17 @@ namespace LinuxSampler {
 
         protected:
 
-            static int getSample(sample_t* src, int pos) {
+            inline static int32_t getSample(sample_t* src, int pos) {
                 if (BITDEPTH24) {
                     pos *= 3;
+                    #if WORDS_BIGENDIAN
                     unsigned char* p = (unsigned char*)src;
                     return p[pos] << 8 | p[pos + 1] << 16 | p[pos + 2] << 24;
+                    #else
+                    // 24bit read optimization: 
+                    // a misaligned 32bit read and subquent 8 bit shift is faster (on x86) than reading 3 single bytes and shifting them
+                    return (*((int32_t *)(&((char *)(src))[pos])))<<8;
+                    #endif
                 } else {
                     return src[pos];
                 }
@@ -166,10 +174,10 @@ namespace LinuxSampler {
                     int x2 = getSample(pSrc, pos_int + 1);
                     float samplePoint  = (x1 + pos_fract * (x2 - x1));
                 #else // polynomial interpolation
-                    float xm1 = pSrc[pos_int];
-                    float x0  = pSrc[pos_int+1];
-                    float x1  = pSrc[pos_int+2];
-                    float x2  = pSrc[pos_int+3];
+                    float xm1 = getSample(pSrc, pos_int);
+                    float x0  = getSample(pSrc, pos_int + 1);
+                    float x1  = getSample(pSrc, pos_int + 2);
+                    float x2  = getSample(pSrc, pos_int + 3);
                     float a   = (3.0f * (x0 - x1) - xm1 + x2) * 0.5f;
                     float b   = 2.0f * x1 + xm1 - (5.0f * x0 + x2) * 0.5f;
                     float c   = (x1 - xm1) * 0.5f;
@@ -198,20 +206,20 @@ namespace LinuxSampler {
                     samplePoint.right = (x1 + pos_fract * (x2 - x1));
                 #else // polynomial interpolation
                     // calculate left channel
-                    float xm1 = pSrc[pos_int];
-                    float x0  = pSrc[pos_int+2];
-                    float x1  = pSrc[pos_int+4];
-                    float x2  = pSrc[pos_int+6];
+                    float xm1 = getSample(pSrc, pos_int);
+                    float x0  = getSample(pSrc, pos_int + 2);
+                    float x1  = getSample(pSrc, pos_int + 4);
+                    float x2  = getSample(pSrc, pos_int + 6);
                     float a   = (3.0f * (x0 - x1) - xm1 + x2) * 0.5f;
                     float b   = 2.0f * x1 + xm1 - (5.0f * x0 + x2) * 0.5f;
                     float c   = (x1 - xm1) * 0.5f;
                     samplePoint.left = (((a * pos_fract) + b) * pos_fract + c) * pos_fract + x0;
 
                     //calculate right channel
-                    xm1 = pSrc[pos_int+1];
-                    x0  = pSrc[pos_int+3];
-                    x1  = pSrc[pos_int+5];
-                    x2  = pSrc[pos_int+7];
+                    xm1 = getSample(pSrc, pos_int + 1);
+                    x0  = getSample(pSrc, pos_int + 3);
+                    x1  = getSample(pSrc, pos_int + 5);
+                    x2  = getSample(pSrc, pos_int + 7);
                     a   = (3.0f * (x0 - x1) - xm1 + x2) * 0.5f;
                     b   = 2.0f * x1 + xm1 - (5.0f * x0 + x2) * 0.5f;
                     c   = (x1 - xm1) * 0.5f;
@@ -222,7 +230,7 @@ namespace LinuxSampler {
                 return samplePoint;
             }
 
-#if CONFIG_ASM && ARCH_X86
+#if 0 // CONFIG_ASM && ARCH_X86
             // TODO: no support for cubic interpolation yet
             inline static void Interpolate4StepsMonoMMXSSE(sample_t* pSrc, void* Pos, float& Pitch) {
                 /* calculate playback position of each of the 4 samples by adding the associated pitch */
