@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 Christian Schoenebeck                              *
+ *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -49,16 +49,25 @@ namespace LinuxSampler { namespace gig {
     void DiskThread::Reset() {
         bool running = this->IsRunning();
         if (running) this->StopThread();
-        for (int i = 0; i < CONFIG_MAX_STREAMS; i++) {
+        for (int i = 0; i < Streams; i++) {
             pStreams[i]->Kill();
         }
-        for (int i = 1; i <= CONFIG_MAX_STREAMS; i++) {
+        for (int i = 1; i <= Streams; i++) {
             pCreatedStreams[i] = NULL;
         }
         GhostQueue->init();
         CreationQueue->init();
         DeletionQueue->init();
-        ActiveStreamCount = 0;
+        DeletionNotificationQueue.init();
+
+        // make sure that all DimensionRegions are released
+        while (DeleteDimregQueue->read_space() > 0) {
+            ::gig::DimensionRegion* dimreg;
+            DeleteDimregQueue->pop(&dimreg);
+            pInstruments->HandBackDimReg(dimreg);
+        }
+        DeleteDimregQueue->init();
+        SetActiveStreamCount(0);
         ActiveStreamCountMax = 0;
         if (running) this->StartThread(); // start thread only if it was running before
     }
@@ -134,10 +143,18 @@ namespace LinuxSampler { namespace gig {
     }
 
     /**
-     * Returns -1 if command queue is full, 0 on success (will be called by audio
-     * thread within the voice class).
+     * Request the disk thread to delete the given disk stream. This method
+     * will return immediately, thus it won't block until the respective voice
+     * was actually deleted. (Called by audio thread within the Voice class).
+     *
+     * @param pStreamRef           - stream that shall be deleted
+     * @param bRequestNotification - set to true in case you want to receive a
+     *                               notification once the stream has actually
+     *                               been deleted
+     * @returns 0 on success, -1 if command queue is full
+     * @see AskForDeletedStream()
      */
-    int DiskThread::OrderDeletionOfStream(Stream::reference_t* pStreamRef) {
+    int DiskThread::OrderDeletionOfStream(Stream::reference_t* pStreamRef, bool bRequestNotification) {
         dmsg(4,("Disk Thread: stream deletion ordered\n"));
         if (DeletionQueue->write_space() < 1) {
             dmsg(1,("DiskThread: Deletion queue full!\n"));
@@ -148,11 +165,47 @@ namespace LinuxSampler { namespace gig {
         cmd.pStream = pStreamRef->pStream;
         cmd.hStream = pStreamRef->hStream;
         cmd.OrderID = pStreamRef->OrderID;
+        cmd.bNotify = bRequestNotification;
 
         DeletionQueue->push(&cmd);
         return 0;
     }
 
+    /**
+     * Tell the disk thread to release a dimension region that belongs
+     * to an instrument which isn't loaded anymore. The disk thread
+     * will hand back the dimension region to the instrument resource
+     * manager. (OrderDeletionOfDimreg is called from the audio thread
+     * when a voice dies.)
+     */
+    int DiskThread::OrderDeletionOfDimreg(::gig::DimensionRegion* dimreg) {
+        dmsg(4,("Disk Thread: dimreg deletion ordered\n"));
+        if (DeleteDimregQueue->write_space() < 1) {
+            dmsg(1,("DiskThread: DeleteDimreg queue full!\n"));
+            return -1;
+        }
+        DeleteDimregQueue->push(&dimreg);
+        return 0;
+    }
+
+    /**
+     * Tell the disk thread to do a program change on the specified
+     * EngineChannel.
+     */
+    int DiskThread::OrderProgramChange(uint8_t Program, EngineChannel* pEngineChannel) {
+        program_change_command_t cmd;
+        cmd.Program = Program;
+        cmd.pEngineChannel = pEngineChannel;
+
+        dmsg(4,("Disk Thread: program change ordered\n"));
+        if (ProgramChangeQueue.write_space() < 1) {
+            dmsg(1,("DiskThread: ProgramChange queue full!\n"));
+            return -1;
+        }
+        ProgramChangeQueue.push(&cmd);
+        return 0;
+    }
+
     /**
      * Returns the pointer to a disk stream if the ordered disk stream
      * represented by the \a StreamOrderID was already activated by the disk
@@ -178,6 +231,22 @@ namespace LinuxSampler { namespace gig {
         return NULL;
     }
 
+    /**
+     * In case the original sender requested a notification with his stream
+     * deletion order, he can use this method to poll if the respective stream
+     * has actually been deleted.
+     *
+     * @returns handle / identifier of the deleted stream, or
+     *          Stream::INVALID_HANDLE if no notification present
+     */
+    Stream::Handle DiskThread::AskForDeletedStream() {
+        if (DeletionNotificationQueue.read_space()) {
+            Stream::Handle hStream;
+            DeletionNotificationQueue.pop(&hStream);
+            return hStream;
+        } else return Stream::INVALID_HANDLE; // no notification received yet
+    }
+
 
 
     // #########################################################################
@@ -185,51 +254,71 @@ namespace LinuxSampler { namespace gig {
     // #         (following code should only be executed by the disk thread)
 
 
-    DiskThread::DiskThread(uint BufferWrapElements) : Thread(true, false, 1, -2) {
+    DiskThread::DiskThread(int MaxStreams, uint BufferWrapElements, InstrumentResourceManager* pInstruments) :
+        Thread(true, false, 1, -2),
+        pInstruments(pInstruments),
+        DeletionNotificationQueue(4*MaxStreams),
+        ProgramChangeQueue(100)
+    {
         DecompressionBuffer = ::gig::Sample::CreateDecompressionBuffer(CONFIG_STREAM_MAX_REFILL_SIZE);
-        CreationQueue       = new RingBuffer<create_command_t>(1024);
-        DeletionQueue       = new RingBuffer<delete_command_t>(1024);
-        GhostQueue          = new RingBuffer<Stream::Handle>(CONFIG_MAX_STREAMS);
-        Streams             = CONFIG_MAX_STREAMS;
+        CreationQueue       = new RingBuffer<create_command_t,false>(4*MaxStreams);
+        DeletionQueue       = new RingBuffer<delete_command_t,false>(4*MaxStreams);
+        GhostQueue          = new RingBuffer<delete_command_t,false>(MaxStreams);
+        DeleteDimregQueue   = new RingBuffer< ::gig::DimensionRegion*,false>(4*MaxStreams);
+        pStreams            = new Stream*[MaxStreams];
+        pCreatedStreams     = new Stream*[MaxStreams + 1];
+        Streams             = MaxStreams;
         RefillStreamsPerRun = CONFIG_REFILL_STREAMS_PER_RUN;
-        for (int i = 0; i < CONFIG_MAX_STREAMS; i++) {
+        for (int i = 0; i < MaxStreams; i++) {
             pStreams[i] = new Stream(&DecompressionBuffer, CONFIG_STREAM_BUFFER_SIZE, BufferWrapElements); // 131072 sample words
         }
-        for (int i = 1; i <= CONFIG_MAX_STREAMS; i++) {
+        for (int i = 1; i <= MaxStreams; i++) {
             pCreatedStreams[i] = NULL;
         }
         ActiveStreamCountMax = 0;
     }
 
     DiskThread::~DiskThread() {
-        for (int i = 0; i < CONFIG_MAX_STREAMS; i++) {
+        for (int i = 0; i < Streams; i++) {
             if (pStreams[i]) delete pStreams[i];
         }
         if (CreationQueue) delete CreationQueue;
         if (DeletionQueue) delete DeletionQueue;
         if (GhostQueue)    delete GhostQueue;
+        if (DeleteDimregQueue) delete DeleteDimregQueue;
+        if (pStreams)        delete[] pStreams;
+        if (pCreatedStreams) delete[] pCreatedStreams;
+
         ::gig::Sample::DestroyDecompressionBuffer(DecompressionBuffer);
     }
 
     int DiskThread::Main() {
         dmsg(3,("Disk thread running\n"));
         while (true) {
+            #if !defined(WIN32)
             pthread_testcancel(); // mandatory for OSX
+            #endif
+            #if CONFIG_PTHREAD_TESTCANCEL
+            TestCancel();
+            #endif
             IsIdle = true; // will be set to false if a stream got filled
 
             // if there are ghost streams, delete them
             for (int i = 0; i < GhostQueue->read_space(); i++) { //FIXME: unefficient
-                Stream::Handle hGhostStream;
-                GhostQueue->pop(&hGhostStream);
+                delete_command_t ghostStream;
+                GhostQueue->pop(&ghostStream);
                 bool found = false;
                 for (int i = 0; i < this->Streams; i++) {
-                    if (pStreams[i]->GetHandle() == hGhostStream) {
+                    if (pStreams[i]->GetHandle() == ghostStream.hStream) {
                         pStreams[i]->Kill();
                         found = true;
+                        // if original sender requested a notification, let him know now
+                        if (ghostStream.bNotify)
+                            DeletionNotificationQueue.push(&ghostStream.hStream);
                         break;
                     }
                 }
-                if (!found) GhostQueue->push(&hGhostStream); // put ghost stream handle back to the queue
+                if (!found) GhostQueue->push(&ghostStream); // put ghost stream handle back to the queue
             }
 
             // if there are creation commands, create new streams
@@ -246,6 +335,20 @@ namespace LinuxSampler { namespace gig {
                 DeleteStream(command);
             }
 
+            // release DimensionRegions that belong to instruments
+            // that are no longer loaded
+            while (DeleteDimregQueue->read_space() > 0) {
+                ::gig::DimensionRegion* dimreg;
+                DeleteDimregQueue->pop(&dimreg);
+                pInstruments->HandBackDimReg(dimreg);
+            }
+
+            // perform MIDI program change commands
+            while (ProgramChangeQueue.read_space() > 0) {
+                program_change_command_t cmd;
+                ProgramChangeQueue.pop(&cmd);
+                cmd.pEngineChannel->ExecuteProgramChange(cmd.Program);
+            }
             RefillStreams(); // refill the most empty streams
 
             // if nothing was done during this iteration (eg no streambuffer
@@ -256,7 +359,7 @@ namespace LinuxSampler { namespace gig {
             for (int i = Streams - 1; i >= 0; i--) {
                 if (pStreams[i]->GetState() != Stream::state_unused) streamsInUsage++;
             }
-            ActiveStreamCount = streamsInUsage;
+            SetActiveStreamCount(streamsInUsage);
             if (streamsInUsage > ActiveStreamCountMax) ActiveStreamCountMax = streamsInUsage;
         }
 
@@ -287,7 +390,10 @@ namespace LinuxSampler { namespace gig {
     }
 
     void DiskThread::DeleteStream(delete_command_t& Command) {
-        if (Command.pStream) Command.pStream->Kill();
+        if (Command.pStream) {
+            Command.pStream->Kill();
+            if (Command.bNotify) DeletionNotificationQueue.push(&Command.hStream);
+        }
         else { // the stream wasn't created by disk thread or picked up by audio thread yet
 
             // if stream was created but not picked up yet
@@ -295,14 +401,22 @@ namespace LinuxSampler { namespace gig {
             if (pStream && pStream != SLOT_RESERVED) {
                 pStream->Kill();
                 pCreatedStreams[Command.OrderID] = NULL; // free slot for new order
+                // if original sender requested a notification, let him know now
+                if (Command.bNotify)
+                    DeletionNotificationQueue.push(&Command.hStream);
                 return;
             }
 
             // the stream was not created yet
             if (GhostQueue->write_space() > 0) {
-                GhostQueue->push(&Command.hStream);
+                GhostQueue->push(&Command);
+            } else { // error, queue full
+                if (Command.bNotify) {
+                    dmsg(1,("DiskThread: GhostQueue full! (might lead to dead lock with instrument editor!)\n"));
+                } else {
+                    dmsg(1,("DiskThread: GhostQueue full!\n"));
+                }
             }
-            else dmsg(1,("DiskThread: GhostQueue full!\n"));
         }
     }
 
@@ -345,9 +459,9 @@ namespace LinuxSampler { namespace gig {
     /// order ID Generator
     Stream::OrderID_t DiskThread::CreateOrderID() {
         static Stream::OrderID_t counter(0);
-        for (int i = 0; i < CONFIG_MAX_STREAMS; i++) {
-            if (counter == CONFIG_MAX_STREAMS) counter = 1; // we use '0' as 'invalid order' only, so we skip 0
-            else                              counter++;
+        for (int i = 0; i < Streams; i++) {
+            if (counter == Streams) counter = 1; // we use '0' as 'invalid order' only, so we skip 0
+            else                    counter++;
             if (!pCreatedStreams[counter]) {
                 pCreatedStreams[counter] = SLOT_RESERVED; // mark this slot as reserved
                 return counter;                           // found empty slot
