@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005, 2006 Christian Schoenebeck                        *
+ *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -38,8 +38,8 @@ namespace LinuxSampler { namespace gig {
         pLFO3 = new LFOSigned(1200.0f); // pitch EG (-1200..+1200 range)
         KeyGroup = 0;
         SynthesisMode = 0; // set all mode bits to 0 first
-        // select synthesis implementation (currently either pure C++ or MMX+SSE(1))
-        #if CONFIG_ASM && ARCH_X86
+        // select synthesis implementation (asm core is not supported ATM)
+        #if 0 // CONFIG_ASM && ARCH_X86
         SYNTHESIS_MODE_SET_IMPLEMENTATION(SynthesisMode, Features::supportsMMX() && Features::supportsSSE());
         #else
         SYNTHESIS_MODE_SET_IMPLEMENTATION(SynthesisMode, false);
@@ -79,6 +79,7 @@ namespace LinuxSampler { namespace gig {
     int Voice::Trigger(EngineChannel* pEngineChannel, Pool<Event>::Iterator& itNoteOnEvent, int PitchBend, ::gig::DimensionRegion* pDimRgn, type_t VoiceType, int iKeyGroup) {
         this->pEngineChannel = pEngineChannel;
         this->pDimRgn        = pDimRgn;
+        Orphan = false;
 
         #if CONFIG_DEVMODE
         if (itNoteOnEvent->FragmentPos() > pEngine->MaxSamplesPerCycle) { // just a sanity check for debugging
@@ -103,7 +104,7 @@ namespace LinuxSampler { namespace gig {
         // -1.0..1.0). For 24 bit, we downscale from int32.
         float volume = velocityAttenuation / (pSample->BitDepth == 16 ? 32768.0f : 32768.0f * 65536.0f);
 
-        volume *= pDimRgn->SampleAttenuation;
+        volume *= pDimRgn->SampleAttenuation * pEngineChannel->GlobalVolume * GLOBAL_VOLUME;
 
         // the volume of release triggered samples depends on note length
         if (Type == type_release_trigger) {
@@ -141,7 +142,7 @@ namespace LinuxSampler { namespace gig {
 
         float subfragmentRate = pEngine->SampleRate / CONFIG_DEFAULT_SUBFRAGMENT_SIZE;
         CrossfadeSmoother.trigger(crossfadeVolume, subfragmentRate);
-        VolumeSmoother.trigger(pEngineChannel->GlobalVolume, subfragmentRate);
+        VolumeSmoother.trigger(pEngineChannel->MidiVolume, subfragmentRate);
         PanLeftSmoother.trigger(pEngineChannel->GlobalPanLeft, subfragmentRate);
         PanRightSmoother.trigger(pEngineChannel->GlobalPanRight, subfragmentRate);
 
@@ -155,7 +156,16 @@ namespace LinuxSampler { namespace gig {
         const DLS::sample_loop_t& loopinfo = pDimRgn->pSampleLoops[0];
 
         if (DiskVoice) { // voice to be streamed from disk
-            MaxRAMPos = cachedsamples - (pEngine->MaxSamplesPerCycle << CONFIG_MAX_PITCH) / pSample->Channels; //TODO: this calculation is too pessimistic and may better be moved to Render() method, so it calculates MaxRAMPos dependent to the current demand of sample points to be rendered (e.g. in case of JACK)
+            if (cachedsamples > (pEngine->MaxSamplesPerCycle << CONFIG_MAX_PITCH)) {
+                MaxRAMPos = cachedsamples - (pEngine->MaxSamplesPerCycle << CONFIG_MAX_PITCH) / pSample->Channels; //TODO: this calculation is too pessimistic and may better be moved to Render() method, so it calculates MaxRAMPos dependent to the current demand of sample points to be rendered (e.g. in case of JACK)
+            } else {
+                // The cache is too small to fit a max sample buffer.
+                // Setting MaxRAMPos to 0 will probably cause a click
+                // in the audio, but it's better than not handling
+                // this case at all, which would have caused the
+                // unsigned MaxRAMPos to be set to a negative number.
+                MaxRAMPos = 0;
+            }
 
             // check if there's a loop defined which completely fits into the cached (RAM) part of the sample
             RAMLoop = (pDimRgn->SampleLoops && (loopinfo.LoopStart + loopinfo.LoopLength) <= MaxRAMPos);
@@ -182,10 +192,16 @@ namespace LinuxSampler { namespace gig {
 
         // calculate initial pitch value
         {
-            double pitchbasecents = pDimRgn->FineTune + (int) pEngine->ScaleTuning[MIDIKey % 12];
-            if (pDimRgn->PitchTrack) pitchbasecents += (MIDIKey - (int) pDimRgn->UnityNote) * 100;
-            this->PitchBase = RTMath::CentsToFreqRatio(pitchbasecents) * (double(pSample->SamplesPerSecond) / double(pEngine->SampleRate));
-            this->PitchBend = RTMath::CentsToFreqRatio(((double) PitchBend / 8192.0) * 200.0); // pitchbend wheel +-2 semitones = 200 cents
+            double pitchbasecents = pEngineChannel->pInstrument->FineTune + pDimRgn->FineTune + pEngine->ScaleTuning[MIDIKey % 12];
+
+            // GSt behaviour: maximum transpose up is 40 semitones. If
+            // MIDI key is more than 40 semitones above unity note,
+            // the transpose is not done.
+            if (pDimRgn->PitchTrack && (MIDIKey - (int) pDimRgn->UnityNote) < 40) pitchbasecents += (MIDIKey - (int) pDimRgn->UnityNote) * 100;
+
+            this->PitchBase = RTMath::CentsToFreqRatioUnlimited(pitchbasecents) * (double(pSample->SamplesPerSecond) / double(pEngine->SampleRate));
+            this->PitchBendRange = 1.0 / 8192.0 * 100.0 * pEngineChannel->pInstrument->PitchbendRange;
+            this->PitchBend = RTMath::CentsToFreqRatio(PitchBend * PitchBendRange);
         }
 
         // the length of the decay and release curves are dependent on the velocity
@@ -241,7 +257,7 @@ namespace LinuxSampler { namespace gig {
         else
 #else
         {
-            float finalVolume = pEngineChannel->GlobalVolume * crossfadeVolume * EG1.getLevel();
+            float finalVolume = pEngineChannel->MidiVolume * crossfadeVolume * EG1.getLevel();
 
             finalSynthesisParameters.fFinalVolumeLeft  = finalVolume * VolumeLeft  * pEngineChannel->GlobalPanLeft;
             finalSynthesisParameters.fFinalVolumeRight = finalVolume * VolumeRight * pEngineChannel->GlobalPanRight;
@@ -514,8 +530,8 @@ namespace LinuxSampler { namespace gig {
             finalSynthesisParameters.filterLeft.SetType(pDimRgn->VCFType);
             finalSynthesisParameters.filterRight.SetType(pDimRgn->VCFType);
             #else // override filter type
-            FilterLeft.SetType(CONFIG_OVERRIDE_FILTER_TYPE);
-            FilterRight.SetType(CONFIG_OVERRIDE_FILTER_TYPE);
+            finalSynthesisParameters.filterLeft.SetType(CONFIG_OVERRIDE_FILTER_TYPE);
+            finalSynthesisParameters.filterRight.SetType(CONFIG_OVERRIDE_FILTER_TYPE);
             #endif // CONFIG_OVERRIDE_FILTER_TYPE
 
             VCFCutoffCtrl.value    = pEngineChannel->ControllerTable[VCFCutoffCtrl.controller];
@@ -719,7 +735,7 @@ namespace LinuxSampler { namespace gig {
                     CrossfadeSmoother.update(Engine::CrossfadeCurve[CrossfadeAttenuation(itEvent->Param.CC.Value)]);
                 }
                 if (itEvent->Param.CC.Controller == 7) { // volume
-                    VolumeSmoother.update(Engine::VolumeCurve[itEvent->Param.CC.Value] * CONFIG_GLOBAL_ATTENUATION);
+                    VolumeSmoother.update(Engine::VolumeCurve[itEvent->Param.CC.Value]);
                 } else if (itEvent->Param.CC.Controller == 10) { // panpot
                     PanLeftSmoother.update(Engine::PanCurve[128 - itEvent->Param.CC.Value]);
                     PanRightSmoother.update(Engine::PanCurve[itEvent->Param.CC.Value]);
@@ -731,9 +747,7 @@ namespace LinuxSampler { namespace gig {
     }
 
     void Voice::processPitchEvent(RTList<Event>::Iterator& itEvent) {
-        const float pitch = RTMath::CentsToFreqRatio(((double) itEvent->Param.Pitch.Pitch / 8192.0) * 200.0); // +-two semitones = +-200 cents
-        finalSynthesisParameters.fFinalPitch *= pitch;
-        PitchBend = pitch;
+        PitchBend = RTMath::CentsToFreqRatio(itEvent->Param.Pitch.Pitch * PitchBendRange);
     }
 
     void Voice::processCutoffEvent(RTList<Event>::Iterator& itEvent) {
@@ -768,33 +782,54 @@ namespace LinuxSampler { namespace gig {
      *  @param Skip    - number of sample points to skip in output buffer
      */
     void Voice::Synthesize(uint Samples, sample_t* pSrc, uint Skip) {
-        finalSynthesisParameters.pOutLeft  = &pEngineChannel->pOutputLeft[Skip];
-        finalSynthesisParameters.pOutRight = &pEngineChannel->pOutputRight[Skip];
+        finalSynthesisParameters.pOutLeft  = &pEngineChannel->pChannelLeft->Buffer()[Skip];
+        finalSynthesisParameters.pOutRight = &pEngineChannel->pChannelRight->Buffer()[Skip];
         finalSynthesisParameters.pSrc      = pSrc;
 
         RTList<Event>::Iterator itCCEvent = pEngineChannel->pEvents->first();
         RTList<Event>::Iterator itNoteEvent = pEngineChannel->pMIDIKeyInfo[MIDIKey].pEvents->first();
 
-        if (Skip) { // skip events that happened before this voice was triggered
+        if (itTriggerEvent) { // skip events that happened before this voice was triggered
             while (itCCEvent && itCCEvent->FragmentPos() <= Skip) ++itCCEvent;
-            while (itNoteEvent && itNoteEvent->FragmentPos() <= Skip) ++itNoteEvent;
+            // we can't simply compare the timestamp here, because note events
+            // might happen on the same time stamp, so we have to deal on the
+            // actual sequence the note events arrived instead (see bug #112)
+            for (; itNoteEvent; ++itNoteEvent) {
+                if (itTriggerEvent == itNoteEvent) {
+                    ++itNoteEvent;
+                    break;
+                }
+            }
         }
 
         uint killPos;
-        if (itKillEvent) killPos = RTMath::Min(itKillEvent->FragmentPos(), pEngine->MaxFadeOutPos);
+        if (itKillEvent) {
+            int maxFadeOutPos = Samples - pEngine->MinFadeOutSamples;
+            if (maxFadeOutPos < 0) {
+                // There's not enough space in buffer to do a fade out
+                // from max volume (this can only happen for audio
+                // drivers that use Samples < MaxSamplesPerCycle).
+                // End the EG1 here, at pos 0, with a shorter max fade
+                // out time.
+                EG1.enterFadeOutStage(Samples / CONFIG_DEFAULT_SUBFRAGMENT_SIZE);
+                itKillEvent = Pool<Event>::Iterator();
+            } else {
+                killPos = RTMath::Min(itKillEvent->FragmentPos(), maxFadeOutPos);
+            }
+        }
 
         uint i = Skip;
         while (i < Samples) {
             int iSubFragmentEnd = RTMath::Min(i + CONFIG_DEFAULT_SUBFRAGMENT_SIZE, Samples);
 
             // initialize all final synthesis parameters
-            finalSynthesisParameters.fFinalPitch = PitchBase * PitchBend;
             fFinalCutoff    = VCFCutoffCtrl.fvalue;
             fFinalResonance = VCFResonanceCtrl.fvalue;
 
             // process MIDI control change and pitchbend events for this subfragment
             processCCEvents(itCCEvent, iSubFragmentEnd);
 
+            finalSynthesisParameters.fFinalPitch = PitchBase * PitchBend;
             float fFinalVolume = VolumeSmoother.render() * CrossfadeSmoother.render();
 #ifdef CONFIG_PROCESS_MUTED_CHANNELS
             if (pEngineChannel->GetMute()) fFinalVolume = 0;
@@ -803,8 +838,11 @@ namespace LinuxSampler { namespace gig {
             // process transition events (note on, note off & sustain pedal)
             processTransitionEvents(itNoteEvent, iSubFragmentEnd);
 
-            // if the voice was killed in this subfragment switch EG1 to fade out stage
-            if (itKillEvent && killPos <= iSubFragmentEnd) {
+            // if the voice was killed in this subfragment, or if the
+            // filter EG is finished, switch EG1 to fade out stage
+            if ((itKillEvent && killPos <= iSubFragmentEnd) ||
+                (SYNTHESIS_MODE_GET_FILTER(SynthesisMode) &&
+                 EG2.getSegmentType() == EGADSR::segment_end)) {
                 EG1.enterFadeOutStage();
                 itKillEvent = Pool<Event>::Iterator();
             }
@@ -839,6 +877,9 @@ namespace LinuxSampler { namespace gig {
             if (bLFO2Enabled) fFinalCutoff *= pLFO2->render();
             if (bLFO3Enabled) finalSynthesisParameters.fFinalPitch *= RTMath::CentsToFreqRatio(pLFO3->render());
 
+            // limit the pitch so we don't read outside the buffer
+            finalSynthesisParameters.fFinalPitch = RTMath::Min(finalSynthesisParameters.fFinalPitch, float(1 << CONFIG_MAX_PITCH));
+
             // if filter enabled then update filter coefficients
             if (SYNTHESIS_MODE_GET_FILTER(SynthesisMode)) {
                 finalSynthesisParameters.filterLeft.SetParameters(fFinalCutoff, fFinalResonance, pEngine->SampleRate);
@@ -917,13 +958,30 @@ namespace LinuxSampler { namespace gig {
      *  fading down the volume level to avoid clicks and regular processing
      *  until the kill event actually occured!
      *
-     *  @see Kill()
+     * If it's necessary to know when the voice's disk stream was actually
+     * deleted, then one can set the optional @a bRequestNotification
+     * parameter and this method will then return the handle of the disk
+     * stream (unique identifier) and one can use this handle to poll the
+     * disk thread if this stream has been deleted. In any case this method
+     * will return immediately and will not block until the stream actually
+     * was deleted.
+     *
+     * @param bRequestNotification - (optional) whether the disk thread shall
+     *                                provide a notification once it deleted
+     *                               the respective disk stream
+     *                               (default=false)
+     * @returns handle to the voice's disk stream or @c Stream::INVALID_HANDLE
+     *          if the voice did not use a disk stream at all
+     * @see Kill()
      */
-    void Voice::KillImmediately() {
+    Stream::Handle Voice::KillImmediately(bool bRequestNotification) {
+        Stream::Handle hStream = Stream::INVALID_HANDLE;
         if (DiskVoice && DiskStreamRef.State != Stream::state_unused) {
-            pDiskThread->OrderDeletionOfStream(&DiskStreamRef);
+            pDiskThread->OrderDeletionOfStream(&DiskStreamRef, bRequestNotification);
+            hStream = DiskStreamRef.hStream;
         }
         Reset();
+        return hStream;
     }
 
     /**
